package net.sf.itcb.addons.monitoring.track;

import java.util.Collections;
import java.util.HashMap;
import java.util.Map;
import java.util.regex.Pattern;
import java.util.regex.PatternSyntaxException;

import net.sf.itcb.addons.monitoring.track.bean.ItcbTrackBean;

import org.aopalliance.intercept.MethodInterceptor;
import org.aopalliance.intercept.MethodInvocation;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * This interceptor's aim is to log time spent in each layer and the result {@link TracerInterceptor#Status}.<br/>
 * It defines a transaction UUID if not already defined for the current thread.<br/>
 * This id will allow the monitoring visual tool to show the spent time in each layer.<br/>
 * 
 * @author Pierre Le Roux
 */

public class TrackerMethodInterceptor implements MethodInterceptor {

	protected final Logger logger = LoggerFactory.getLogger(getClass());
	
	protected MonitoringTransactionManager monitoringTransactionManager;
	
	protected Map<Pattern, String> patternsLayer;
	
	private Map<String, String> classLayer = Collections.synchronizedMap(new HashMap<String, String>());
	
	public void setMonitoringTransactionManager(MonitoringTransactionManager monitoringTransactionManager) {
		this.monitoringTransactionManager=monitoringTransactionManager;
	}
	


	public void setLayerPatterns(Map<String, String> layerPatterns) {
		this.patternsLayer=compilePatterns(layerPatterns);
	}	
	
	@Override
	public Object invoke(MethodInvocation invocation) throws Throwable {
		logger.debug("track layer");
		long startTime = System.currentTimeMillis();
		ItcbTrackBean itcbTrackBean = new ItcbTrackBean();
		itcbTrackBean.setStartTime(startTime);

		monitoringTransactionManager.setCurrentTrackBean(itcbTrackBean);
		try {
			return invocation.proceed();
	    }
	    catch(Throwable t) {
	    	itcbTrackBean.setException(t);
	    	throw t;
	    }
	    finally {
	    	itcbTrackBean.setTimeInMillis(System.currentTimeMillis() - startTime);
	    	itcbTrackBean.setClassName(invocation.getThis().getClass().getName());
	    	itcbTrackBean.setLayerType(getLayerType(itcbTrackBean.getClassName()));
	    	itcbTrackBean.setMethod(invocation.getMethod().getName());
	    	itcbTrackBean.setParameters(invocation.getArguments());
	    	monitoringTransactionManager.endCurrentTrackBean();
	     }
	}
	
	
	private String getLayerType(String className) {
		String layer = classLayer.get(className);
		if(layer == null) {
			for (Map.Entry<Pattern, String> patternLayer : patternsLayer.entrySet()) {
				if(patternLayer.getKey().matcher(className).matches()) {
					layer =  patternLayer.getValue();
					classLayer.put(className, layer);
					break;
				}
			}
		}
		return layer;
	}



	/**
	 * Compiles the supplied <code>String[]</code> into an array of
	 * {@link Pattern} objects and returns that array.
	 */
	private Map<Pattern, String> compilePatterns(Map<String,String> layerPatterns) throws PatternSyntaxException {
		Map<Pattern, String> destination = new HashMap<Pattern, String>();
		for (Map.Entry<String, String> layerPattern : layerPatterns.entrySet()) {
			destination.put( Pattern.compile(layerPattern.getValue()), layerPattern.getKey());
		}

		return destination;
	}
	
}